/*
   (c) Copyright 2001-2011  The world wide DirectFB Open Source Community (directfb.org)
   (c) Copyright 2000-2004  Convergence (integrated media) GmbH

   All rights reserved.

   Written by Denis Oliver Kropp <dok@directfb.org>,
              Andreas Hundt <andi@fischlustig.de>,
              Sven Neumann <neo@directfb.org>,
              Ville Syrjälä <syrjala@sci.fi> and
              Claudio Ciccani <klan@users.sf.net>.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <config.h>

#include "SaWManManager.h"

#include <directfb_util.h>

#include <direct/debug.h>
#include <direct/mem.h>
#include <direct/memcpy.h>
#include <direct/messages.h>

#include <fusion/conf.h>

#include <core/core.h>

D_DEBUG_DOMAIN( DirectFB_SaWManManager, "DirectFB/SaWManManager", "DirectFB SaWManManager" );

/*********************************************************************************************************************/

DFBResult
SaWManManager_Activate(
                    SaWManManager                             *obj

)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return ISaWManManager_Real__Activate( obj );
    }

    return ISaWManManager_Requestor__Activate( obj );
}

DFBResult
SaWManManager_QueueUpdate(
                    SaWManManager                             *obj,
                    DFBWindowStackingClass                     stacking,
                    const DFBRegion                           *update
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return ISaWManManager_Real__QueueUpdate( obj, stacking, update );
    }

    return ISaWManManager_Requestor__QueueUpdate( obj, stacking, update );
}

DFBResult
SaWManManager_ProcessUpdates(
                    SaWManManager                             *obj,
                    DFBSurfaceFlipFlags                        flags
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return ISaWManManager_Real__ProcessUpdates( obj, flags );
    }

    return ISaWManManager_Requestor__ProcessUpdates( obj, flags );
}

DFBResult
SaWManManager_CloseWindow(
                    SaWManManager                             *obj,
                    SaWManWindow                              *window
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return ISaWManManager_Real__CloseWindow( obj, window );
    }

    return ISaWManManager_Requestor__CloseWindow( obj, window );
}

DFBResult
SaWManManager_InsertWindow(
                    SaWManManager                             *obj,
                    SaWManWindow                              *window,
                    SaWManWindow                              *relative,
                    SaWManWindowRelation                       relation
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return ISaWManManager_Real__InsertWindow( obj, window, relative, relation );
    }

    return ISaWManManager_Requestor__InsertWindow( obj, window, relative, relation );
}

DFBResult
SaWManManager_RemoveWindow(
                    SaWManManager                             *obj,
                    SaWManWindow                              *window
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return ISaWManManager_Real__RemoveWindow( obj, window );
    }

    return ISaWManManager_Requestor__RemoveWindow( obj, window );
}

DFBResult
SaWManManager_SwitchFocus(
                    SaWManManager                             *obj,
                    SaWManWindow                              *window
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return ISaWManManager_Real__SwitchFocus( obj, window );
    }

    return ISaWManManager_Requestor__SwitchFocus( obj, window );
}

DFBResult
SaWManManager_SetScalingMode(
                    SaWManManager                             *obj,
                    SaWManScalingMode                          mode
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return ISaWManManager_Real__SetScalingMode( obj, mode );
    }

    return ISaWManManager_Requestor__SetScalingMode( obj, mode );
}

DFBResult
SaWManManager_SetWindowConfig(
                    SaWManManager                             *obj,
                    SaWManWindow                              *window,
                    const SaWManWindowConfig                  *config,
                    SaWManWindowConfigFlags                    flags
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return ISaWManManager_Real__SetWindowConfig( obj, window, config, flags );
    }

    return ISaWManManager_Requestor__SetWindowConfig( obj, window, config, flags );
}

DFBResult
SaWManManager_IsShowingWindow(
                    SaWManManager                             *obj,
                    SaWManWindow                              *window,
                    DFBBoolean                                *ret_showing
)
{
    if (!fusion_config->secure_fusion || dfb_core_is_master( core_dfb )) {
        return ISaWManManager_Real__IsShowingWindow( obj, window, ret_showing );
    }

    return ISaWManManager_Requestor__IsShowingWindow( obj, window, ret_showing );
}

/*********************************************************************************************************************/

static FusionCallHandlerResult
SaWManManager_Dispatch( int           caller,   /* fusion id of the caller */
                     int           call_arg, /* optional call parameter */
                     void         *ptr, /* optional call parameter */
                     unsigned int  length,
                     void         *ctx,      /* optional handler context */
                     unsigned int  serial,
                     void         *ret_ptr,
                     unsigned int  ret_size,
                     unsigned int *ret_length )
{
    SaWManManager *obj = (SaWManManager*) ctx;
    SaWManManagerDispatch__Dispatch( obj, caller, call_arg, ptr, length, ret_ptr, ret_size, ret_length );

    return FCHR_RETURN;
}

void SaWManManager_Init_Dispatch(
                    CoreDFB              *core,
                    SaWManManager        *obj,
                    FusionCall           *call
)
{
    fusion_call_init3( call, SaWManManager_Dispatch, obj, core->world );
}

void  SaWManManager_Deinit_Dispatch(
                    FusionCall           *call
)
{
     fusion_call_destroy( call );
}

/*********************************************************************************************************************/


DFBResult
ISaWManManager_Requestor__Activate( SaWManManager *obj

)
{
    DFBResult           ret;
    SaWManManagerActivate       *args = (SaWManManagerActivate*) alloca( sizeof(SaWManManagerActivate) );
    SaWManManagerActivateReturn *return_args = (SaWManManagerActivateReturn*) alloca( sizeof(SaWManManagerActivateReturn) );

    D_DEBUG_AT( DirectFB_SaWManManager, "ISaWManManager_Requestor::%s()\n", __FUNCTION__ );



    ret = (DFBResult) SaWManManager_Call( obj, FCEF_NONE, _SaWManManager_Activate, args, sizeof(SaWManManagerActivate), return_args, sizeof(SaWManManagerActivateReturn), NULL );
    if (ret) {
        D_DERROR( ret, "%s: SaWManManager_Call( SaWManManager_Activate ) failed!\n", __FUNCTION__ );
        return ret;
    }

    if (return_args->result) {
         /*D_DERROR( return_args->result, "%s: SaWManManager_Activate failed!\n", __FUNCTION__ );*/
         return return_args->result;
    }


    return DFB_OK;
}


DFBResult
ISaWManManager_Requestor__QueueUpdate( SaWManManager *obj,
                    DFBWindowStackingClass                     stacking,
                    const DFBRegion                           *update
)
{
    DFBResult           ret;
    SaWManManagerQueueUpdate       *args = (SaWManManagerQueueUpdate*) alloca( sizeof(SaWManManagerQueueUpdate) );

    D_DEBUG_AT( DirectFB_SaWManManager, "ISaWManManager_Requestor::%s()\n", __FUNCTION__ );


    args->stacking = stacking;
  if (update) {
    args->update = *update;
    args->update_set = true;
  }
  else
    args->update_set = false;

    ret = (DFBResult) SaWManManager_Call( obj, (FusionCallExecFlags)(FCEF_ONEWAY), _SaWManManager_QueueUpdate, args, sizeof(SaWManManagerQueueUpdate), NULL, 0, NULL );
    if (ret) {
        D_DERROR( ret, "%s: SaWManManager_Call( SaWManManager_QueueUpdate ) failed!\n", __FUNCTION__ );
        return ret;
    }


    return DFB_OK;
}


DFBResult
ISaWManManager_Requestor__ProcessUpdates( SaWManManager *obj,
                    DFBSurfaceFlipFlags                        flags
)
{
    DFBResult           ret;
    SaWManManagerProcessUpdates       *args = (SaWManManagerProcessUpdates*) alloca( sizeof(SaWManManagerProcessUpdates) );

    D_DEBUG_AT( DirectFB_SaWManManager, "ISaWManManager_Requestor::%s()\n", __FUNCTION__ );


    args->flags = flags;

    ret = (DFBResult) SaWManManager_Call( obj, (FusionCallExecFlags)(FCEF_ONEWAY), _SaWManManager_ProcessUpdates, args, sizeof(SaWManManagerProcessUpdates), NULL, 0, NULL );
    if (ret) {
        D_DERROR( ret, "%s: SaWManManager_Call( SaWManManager_ProcessUpdates ) failed!\n", __FUNCTION__ );
        return ret;
    }


    return DFB_OK;
}


DFBResult
ISaWManManager_Requestor__CloseWindow( SaWManManager *obj,
                    SaWManWindow                              *window
)
{
    DFBResult           ret;
    SaWManManagerCloseWindow       *args = (SaWManManagerCloseWindow*) alloca( sizeof(SaWManManagerCloseWindow) );

    D_DEBUG_AT( DirectFB_SaWManManager, "ISaWManManager_Requestor::%s()\n", __FUNCTION__ );

    D_ASSERT( window != NULL );

    args->window_id = SaWManWindow_GetID( window );

    ret = (DFBResult) SaWManManager_Call( obj, (FusionCallExecFlags)(FCEF_ONEWAY), _SaWManManager_CloseWindow, args, sizeof(SaWManManagerCloseWindow), NULL, 0, NULL );
    if (ret) {
        D_DERROR( ret, "%s: SaWManManager_Call( SaWManManager_CloseWindow ) failed!\n", __FUNCTION__ );
        return ret;
    }


    return DFB_OK;
}


DFBResult
ISaWManManager_Requestor__InsertWindow( SaWManManager *obj,
                    SaWManWindow                              *window,
                    SaWManWindow                              *relative,
                    SaWManWindowRelation                       relation
)
{
    DFBResult           ret;
    SaWManManagerInsertWindow       *args = (SaWManManagerInsertWindow*) alloca( sizeof(SaWManManagerInsertWindow) );

    D_DEBUG_AT( DirectFB_SaWManManager, "ISaWManManager_Requestor::%s()\n", __FUNCTION__ );

    D_ASSERT( window != NULL );

    args->window_id = SaWManWindow_GetID( window );
  if (relative) {
    args->relative_id = SaWManWindow_GetID( relative );
    args->relative_set = true;
  }
  else
    args->relative_set = false;
    args->relation = relation;

    ret = (DFBResult) SaWManManager_Call( obj, (FusionCallExecFlags)(FCEF_ONEWAY), _SaWManManager_InsertWindow, args, sizeof(SaWManManagerInsertWindow), NULL, 0, NULL );
    if (ret) {
        D_DERROR( ret, "%s: SaWManManager_Call( SaWManManager_InsertWindow ) failed!\n", __FUNCTION__ );
        return ret;
    }


    return DFB_OK;
}


DFBResult
ISaWManManager_Requestor__RemoveWindow( SaWManManager *obj,
                    SaWManWindow                              *window
)
{
    DFBResult           ret;
    SaWManManagerRemoveWindow       *args = (SaWManManagerRemoveWindow*) alloca( sizeof(SaWManManagerRemoveWindow) );

    D_DEBUG_AT( DirectFB_SaWManManager, "ISaWManManager_Requestor::%s()\n", __FUNCTION__ );

    D_ASSERT( window != NULL );

    args->window_id = SaWManWindow_GetID( window );

    ret = (DFBResult) SaWManManager_Call( obj, (FusionCallExecFlags)(FCEF_ONEWAY), _SaWManManager_RemoveWindow, args, sizeof(SaWManManagerRemoveWindow), NULL, 0, NULL );
    if (ret) {
        D_DERROR( ret, "%s: SaWManManager_Call( SaWManManager_RemoveWindow ) failed!\n", __FUNCTION__ );
        return ret;
    }


    return DFB_OK;
}


DFBResult
ISaWManManager_Requestor__SwitchFocus( SaWManManager *obj,
                    SaWManWindow                              *window
)
{
    DFBResult           ret;
    SaWManManagerSwitchFocus       *args = (SaWManManagerSwitchFocus*) alloca( sizeof(SaWManManagerSwitchFocus) );

    D_DEBUG_AT( DirectFB_SaWManManager, "ISaWManManager_Requestor::%s()\n", __FUNCTION__ );

    D_ASSERT( window != NULL );

    args->window_id = SaWManWindow_GetID( window );

    ret = (DFBResult) SaWManManager_Call( obj, (FusionCallExecFlags)(FCEF_ONEWAY), _SaWManManager_SwitchFocus, args, sizeof(SaWManManagerSwitchFocus), NULL, 0, NULL );
    if (ret) {
        D_DERROR( ret, "%s: SaWManManager_Call( SaWManManager_SwitchFocus ) failed!\n", __FUNCTION__ );
        return ret;
    }


    return DFB_OK;
}


DFBResult
ISaWManManager_Requestor__SetScalingMode( SaWManManager *obj,
                    SaWManScalingMode                          mode
)
{
    DFBResult           ret;
    SaWManManagerSetScalingMode       *args = (SaWManManagerSetScalingMode*) alloca( sizeof(SaWManManagerSetScalingMode) );

    D_DEBUG_AT( DirectFB_SaWManManager, "ISaWManManager_Requestor::%s()\n", __FUNCTION__ );


    args->mode = mode;

    ret = (DFBResult) SaWManManager_Call( obj, (FusionCallExecFlags)(FCEF_ONEWAY), _SaWManManager_SetScalingMode, args, sizeof(SaWManManagerSetScalingMode), NULL, 0, NULL );
    if (ret) {
        D_DERROR( ret, "%s: SaWManManager_Call( SaWManManager_SetScalingMode ) failed!\n", __FUNCTION__ );
        return ret;
    }


    return DFB_OK;
}


DFBResult
ISaWManManager_Requestor__SetWindowConfig( SaWManManager *obj,
                    SaWManWindow                              *window,
                    const SaWManWindowConfig                  *config,
                    SaWManWindowConfigFlags                    flags
)
{
    DFBResult           ret;
    SaWManManagerSetWindowConfig       *args = (SaWManManagerSetWindowConfig*) alloca( sizeof(SaWManManagerSetWindowConfig) );

    D_DEBUG_AT( DirectFB_SaWManManager, "ISaWManManager_Requestor::%s()\n", __FUNCTION__ );

    D_ASSERT( window != NULL );
    D_ASSERT( config != NULL );

    args->window_id = SaWManWindow_GetID( window );
    args->config = *config;
    args->flags = flags;

    ret = (DFBResult) SaWManManager_Call( obj, (FusionCallExecFlags)(FCEF_ONEWAY), _SaWManManager_SetWindowConfig, args, sizeof(SaWManManagerSetWindowConfig), NULL, 0, NULL );
    if (ret) {
        D_DERROR( ret, "%s: SaWManManager_Call( SaWManManager_SetWindowConfig ) failed!\n", __FUNCTION__ );
        return ret;
    }


    return DFB_OK;
}


DFBResult
ISaWManManager_Requestor__IsShowingWindow( SaWManManager *obj,
                    SaWManWindow                              *window,
                    DFBBoolean                                *ret_showing
)
{
    DFBResult           ret;
    SaWManManagerIsShowingWindow       *args = (SaWManManagerIsShowingWindow*) alloca( sizeof(SaWManManagerIsShowingWindow) );
    SaWManManagerIsShowingWindowReturn *return_args = (SaWManManagerIsShowingWindowReturn*) alloca( sizeof(SaWManManagerIsShowingWindowReturn) );

    D_DEBUG_AT( DirectFB_SaWManManager, "ISaWManManager_Requestor::%s()\n", __FUNCTION__ );

    D_ASSERT( window != NULL );

    args->window_id = SaWManWindow_GetID( window );

    ret = (DFBResult) SaWManManager_Call( obj, FCEF_NONE, _SaWManManager_IsShowingWindow, args, sizeof(SaWManManagerIsShowingWindow), return_args, sizeof(SaWManManagerIsShowingWindowReturn), NULL );
    if (ret) {
        D_DERROR( ret, "%s: SaWManManager_Call( SaWManManager_IsShowingWindow ) failed!\n", __FUNCTION__ );
        return ret;
    }

    if (return_args->result) {
         /*D_DERROR( return_args->result, "%s: SaWManManager_IsShowingWindow failed!\n", __FUNCTION__ );*/
         return return_args->result;
    }

    *ret_showing = return_args->showing;

    return DFB_OK;
}

/*********************************************************************************************************************/

static DFBResult
__SaWManManagerDispatch__Dispatch( SaWManManager *obj,
                                FusionID      caller,
                                int           method,
                                void         *ptr,
                                unsigned int  length,
                                void         *ret_ptr,
                                unsigned int  ret_size,
                                unsigned int *ret_length )
{
    D_UNUSED
    DFBResult ret;


    switch (method) {
        case _SaWManManager_Activate: {
            D_UNUSED
            SaWManManagerActivate       *args        = (SaWManManagerActivate *) ptr;
            SaWManManagerActivateReturn *return_args = (SaWManManagerActivateReturn *) ret_ptr;

            D_DEBUG_AT( DirectFB_SaWManManager, "=-> SaWManManager_Activate\n" );

            return_args->result = ISaWManManager_Real__Activate( obj );
            if (return_args->result == DFB_OK) {
            }

            *ret_length = sizeof(SaWManManagerActivateReturn);

            return DFB_OK;
        }

        case _SaWManManager_QueueUpdate: {
            D_UNUSED
            SaWManManagerQueueUpdate       *args        = (SaWManManagerQueueUpdate *) ptr;

            D_DEBUG_AT( DirectFB_SaWManManager, "=-> SaWManManager_QueueUpdate\n" );

            D_DEBUG_AT( DirectFB_SaWManManager, "  -> stacking = %x\n", args->stacking );
  if (args->update_set)
         ;    // TODO: DFBRegion_debug args->update;

            ISaWManManager_Real__QueueUpdate( obj, args->stacking, args->update_set ? &args->update : NULL );

            return DFB_OK;
        }

        case _SaWManManager_ProcessUpdates: {
            D_UNUSED
            SaWManManagerProcessUpdates       *args        = (SaWManManagerProcessUpdates *) ptr;

            D_DEBUG_AT( DirectFB_SaWManManager, "=-> SaWManManager_ProcessUpdates\n" );

            D_DEBUG_AT( DirectFB_SaWManManager, "  -> flags = %x\n", args->flags );

            ISaWManManager_Real__ProcessUpdates( obj, args->flags );

            return DFB_OK;
        }

        case _SaWManManager_CloseWindow: {
    SaWManWindow *window = NULL;
            D_UNUSED
            SaWManManagerCloseWindow       *args        = (SaWManManagerCloseWindow *) ptr;

            D_DEBUG_AT( DirectFB_SaWManManager, "=-> SaWManManager_CloseWindow\n" );

            D_DEBUG_AT( DirectFB_SaWManManager, "  -> window = %d\n", args->window_id );

            ret = (DFBResult) SaWManWindow_Lookup( core_dfb, args->window_id, caller, &window );
            if (ret) {
                 D_DERROR( ret, "%s: Looking up window by ID %u failed!\n", __FUNCTION__, args->window_id );
                 return DFB_OK;
            }

            ISaWManManager_Real__CloseWindow( obj, window );

            if (window)
                SaWManWindow_Unref( window );

            return DFB_OK;
        }

        case _SaWManManager_InsertWindow: {
    SaWManWindow *window = NULL;
    SaWManWindow *relative = NULL;
            D_UNUSED
            SaWManManagerInsertWindow       *args        = (SaWManManagerInsertWindow *) ptr;

            D_DEBUG_AT( DirectFB_SaWManManager, "=-> SaWManManager_InsertWindow\n" );

            D_DEBUG_AT( DirectFB_SaWManManager, "  -> window = %d\n", args->window_id );
  if (args->relative_set)
            D_DEBUG_AT( DirectFB_SaWManManager, "  -> relative = %d\n", args->relative_id );
            D_DEBUG_AT( DirectFB_SaWManManager, "  -> relation = %x\n", args->relation );

            ret = (DFBResult) SaWManWindow_Lookup( core_dfb, args->window_id, caller, &window );
            if (ret) {
                 D_DERROR( ret, "%s: Looking up window by ID %u failed!\n", __FUNCTION__, args->window_id );
                 return DFB_OK;
            }

            if (args->relative_set) {
                ret = (DFBResult) SaWManWindow_Lookup( core_dfb, args->relative_id, caller, &relative );
                if (ret) {
                     D_DERROR( ret, "%s: Looking up relative by ID %u failed!\n", __FUNCTION__, args->relative_id );
                     return DFB_OK;
                }
            }

            ISaWManManager_Real__InsertWindow( obj, window, args->relative_set ? relative : NULL, args->relation );

            if (window)
                SaWManWindow_Unref( window );

            if (relative)
                SaWManWindow_Unref( relative );

            return DFB_OK;
        }

        case _SaWManManager_RemoveWindow: {
    SaWManWindow *window = NULL;
            D_UNUSED
            SaWManManagerRemoveWindow       *args        = (SaWManManagerRemoveWindow *) ptr;

            D_DEBUG_AT( DirectFB_SaWManManager, "=-> SaWManManager_RemoveWindow\n" );

            D_DEBUG_AT( DirectFB_SaWManManager, "  -> window = %d\n", args->window_id );

            ret = (DFBResult) SaWManWindow_Lookup( core_dfb, args->window_id, caller, &window );
            if (ret) {
                 D_DERROR( ret, "%s: Looking up window by ID %u failed!\n", __FUNCTION__, args->window_id );
                 return DFB_OK;
            }

            ISaWManManager_Real__RemoveWindow( obj, window );

            if (window)
                SaWManWindow_Unref( window );

            return DFB_OK;
        }

        case _SaWManManager_SwitchFocus: {
    SaWManWindow *window = NULL;
            D_UNUSED
            SaWManManagerSwitchFocus       *args        = (SaWManManagerSwitchFocus *) ptr;

            D_DEBUG_AT( DirectFB_SaWManManager, "=-> SaWManManager_SwitchFocus\n" );

            D_DEBUG_AT( DirectFB_SaWManManager, "  -> window = %d\n", args->window_id );

            ret = (DFBResult) SaWManWindow_Lookup( core_dfb, args->window_id, caller, &window );
            if (ret) {
                 D_DERROR( ret, "%s: Looking up window by ID %u failed!\n", __FUNCTION__, args->window_id );
                 return DFB_OK;
            }

            ISaWManManager_Real__SwitchFocus( obj, window );

            if (window)
                SaWManWindow_Unref( window );

            return DFB_OK;
        }

        case _SaWManManager_SetScalingMode: {
            D_UNUSED
            SaWManManagerSetScalingMode       *args        = (SaWManManagerSetScalingMode *) ptr;

            D_DEBUG_AT( DirectFB_SaWManManager, "=-> SaWManManager_SetScalingMode\n" );

            D_DEBUG_AT( DirectFB_SaWManManager, "  -> mode = %x\n", args->mode );

            ISaWManManager_Real__SetScalingMode( obj, args->mode );

            return DFB_OK;
        }

        case _SaWManManager_SetWindowConfig: {
    SaWManWindow *window = NULL;
            D_UNUSED
            SaWManManagerSetWindowConfig       *args        = (SaWManManagerSetWindowConfig *) ptr;

            D_DEBUG_AT( DirectFB_SaWManManager, "=-> SaWManManager_SetWindowConfig\n" );

            D_DEBUG_AT( DirectFB_SaWManManager, "  -> window = %d\n", args->window_id );
         ;    // TODO: SaWManWindowConfig_debug args->config;
            D_DEBUG_AT( DirectFB_SaWManManager, "  -> flags = %x\n", args->flags );

            ret = (DFBResult) SaWManWindow_Lookup( core_dfb, args->window_id, caller, &window );
            if (ret) {
                 D_DERROR( ret, "%s: Looking up window by ID %u failed!\n", __FUNCTION__, args->window_id );
                 return DFB_OK;
            }

            ISaWManManager_Real__SetWindowConfig( obj, window, &args->config, args->flags );

            if (window)
                SaWManWindow_Unref( window );

            return DFB_OK;
        }

        case _SaWManManager_IsShowingWindow: {
    SaWManWindow *window = NULL;
            D_UNUSED
            SaWManManagerIsShowingWindow       *args        = (SaWManManagerIsShowingWindow *) ptr;
            SaWManManagerIsShowingWindowReturn *return_args = (SaWManManagerIsShowingWindowReturn *) ret_ptr;

            D_DEBUG_AT( DirectFB_SaWManManager, "=-> SaWManManager_IsShowingWindow\n" );

            ret = (DFBResult) SaWManWindow_Lookup( core_dfb, args->window_id, caller, &window );
            if (ret) {
                 D_DERROR( ret, "%s: Looking up window by ID %u failed!\n", __FUNCTION__, args->window_id );
                 return_args->result = ret;
                 return DFB_OK;
            }

            return_args->result = ISaWManManager_Real__IsShowingWindow( obj, window, &return_args->showing );
            if (return_args->result == DFB_OK) {
            }

            *ret_length = sizeof(SaWManManagerIsShowingWindowReturn);

            if (window)
                SaWManWindow_Unref( window );

            return DFB_OK;
        }

    }

    return DFB_NOSUCHMETHOD;
}
/*********************************************************************************************************************/

DFBResult
SaWManManagerDispatch__Dispatch( SaWManManager *obj,
                                FusionID      caller,
                                int           method,
                                void         *ptr,
                                unsigned int  length,
                                void         *ret_ptr,
                                unsigned int  ret_size,
                                unsigned int *ret_length )
{
    DFBResult ret;

    D_DEBUG_AT( DirectFB_SaWManManager, "SaWManManagerDispatch::%s( %p )\n", __FUNCTION__, obj );

    ret = __SaWManManagerDispatch__Dispatch( obj, caller, method, ptr, length, ret_ptr, ret_size, ret_length );

    return ret;
}
