/*
   (c) Copyright 2001-2011  The world wide DirectFB Open Source Community (directfb.org)
   (c) Copyright 2000-2004  Convergence (integrated media) GmbH

   All rights reserved.

   Written by Denis Oliver Kropp <dok@directfb.org>,
              Andreas Hundt <andi@fischlustig.de>,
              Sven Neumann <neo@directfb.org>,
              Ville Syrjälä <syrjala@sci.fi> and
              Claudio Ciccani <klan@users.sf.net>.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef ___SaWManManager__H___
#define ___SaWManManager__H___

#include "SaWManManager_includes.h"

/**********************************************************************************************************************
 * SaWManManager
 */

#ifdef __cplusplus
#include <core/Interface.h>

extern "C" {
#endif


DFBResult SaWManManager_Activate(
                    SaWManManager                             *obj
);

DFBResult SaWManManager_QueueUpdate(
                    SaWManManager                             *obj,
                    DFBWindowStackingClass                     stacking,
                    const DFBRegion                           *update);

DFBResult SaWManManager_ProcessUpdates(
                    SaWManManager                             *obj,
                    DFBSurfaceFlipFlags                        flags);

DFBResult SaWManManager_CloseWindow(
                    SaWManManager                             *obj,
                    SaWManWindow                              *window);

DFBResult SaWManManager_InsertWindow(
                    SaWManManager                             *obj,
                    SaWManWindow                              *window,
                    SaWManWindow                              *relative,
                    SaWManWindowRelation                       relation);

DFBResult SaWManManager_RemoveWindow(
                    SaWManManager                             *obj,
                    SaWManWindow                              *window);

DFBResult SaWManManager_SwitchFocus(
                    SaWManManager                             *obj,
                    SaWManWindow                              *window);

DFBResult SaWManManager_SetScalingMode(
                    SaWManManager                             *obj,
                    SaWManScalingMode                          mode);

DFBResult SaWManManager_SetWindowConfig(
                    SaWManManager                             *obj,
                    SaWManWindow                              *window,
                    const SaWManWindowConfig                  *config,
                    SaWManWindowConfigFlags                    flags);

DFBResult SaWManManager_IsShowingWindow(
                    SaWManManager                             *obj,
                    SaWManWindow                              *window,
                    DFBBoolean                                *ret_showing);


void SaWManManager_Init_Dispatch(
                    CoreDFB              *core,
                    SaWManManager        *obj,
                    FusionCall           *call
);

void  SaWManManager_Deinit_Dispatch(
                    FusionCall           *call
);


#ifdef __cplusplus
}
#endif




/*
 * SaWManManager Calls
 */
typedef enum {
    _SaWManManager_Activate = 1,
    _SaWManManager_QueueUpdate = 2,
    _SaWManManager_ProcessUpdates = 3,
    _SaWManManager_CloseWindow = 4,
    _SaWManManager_InsertWindow = 5,
    _SaWManManager_RemoveWindow = 6,
    _SaWManManager_SwitchFocus = 7,
    _SaWManManager_SetScalingMode = 8,
    _SaWManManager_SetWindowConfig = 9,
    _SaWManManager_IsShowingWindow = 10,
} SaWManManagerCall;

/*
 * SaWManManager_Activate
 */
typedef struct {
} SaWManManagerActivate;

typedef struct {
    DFBResult                                  result;
} SaWManManagerActivateReturn;


/*
 * SaWManManager_QueueUpdate
 */
typedef struct {
    DFBWindowStackingClass                     stacking;
    bool                                       update_set;
    DFBRegion                                  update;
} SaWManManagerQueueUpdate;

typedef struct {
    DFBResult                                  result;
} SaWManManagerQueueUpdateReturn;


/*
 * SaWManManager_ProcessUpdates
 */
typedef struct {
    DFBSurfaceFlipFlags                        flags;
} SaWManManagerProcessUpdates;

typedef struct {
    DFBResult                                  result;
} SaWManManagerProcessUpdatesReturn;


/*
 * SaWManManager_CloseWindow
 */
typedef struct {
    u32                                        window_id;
} SaWManManagerCloseWindow;

typedef struct {
    DFBResult                                  result;
} SaWManManagerCloseWindowReturn;


/*
 * SaWManManager_InsertWindow
 */
typedef struct {
    u32                                        window_id;
    bool                                       relative_set;
    u32                                        relative_id;
    SaWManWindowRelation                       relation;
} SaWManManagerInsertWindow;

typedef struct {
    DFBResult                                  result;
} SaWManManagerInsertWindowReturn;


/*
 * SaWManManager_RemoveWindow
 */
typedef struct {
    u32                                        window_id;
} SaWManManagerRemoveWindow;

typedef struct {
    DFBResult                                  result;
} SaWManManagerRemoveWindowReturn;


/*
 * SaWManManager_SwitchFocus
 */
typedef struct {
    u32                                        window_id;
} SaWManManagerSwitchFocus;

typedef struct {
    DFBResult                                  result;
} SaWManManagerSwitchFocusReturn;


/*
 * SaWManManager_SetScalingMode
 */
typedef struct {
    SaWManScalingMode                          mode;
} SaWManManagerSetScalingMode;

typedef struct {
    DFBResult                                  result;
} SaWManManagerSetScalingModeReturn;


/*
 * SaWManManager_SetWindowConfig
 */
typedef struct {
    u32                                        window_id;
    SaWManWindowConfig                         config;
    SaWManWindowConfigFlags                    flags;
} SaWManManagerSetWindowConfig;

typedef struct {
    DFBResult                                  result;
} SaWManManagerSetWindowConfigReturn;


/*
 * SaWManManager_IsShowingWindow
 */
typedef struct {
    u32                                        window_id;
} SaWManManagerIsShowingWindow;

typedef struct {
    DFBResult                                  result;
    DFBBoolean                                 showing;
} SaWManManagerIsShowingWindowReturn;


DFBResult ISaWManManager_Real__Activate( SaWManManager *obj
 );

DFBResult ISaWManManager_Real__QueueUpdate( SaWManManager *obj,
                    DFBWindowStackingClass                     stacking,
                    const DFBRegion                           *update );

DFBResult ISaWManManager_Real__ProcessUpdates( SaWManManager *obj,
                    DFBSurfaceFlipFlags                        flags );

DFBResult ISaWManManager_Real__CloseWindow( SaWManManager *obj,
                    SaWManWindow                              *window );

DFBResult ISaWManManager_Real__InsertWindow( SaWManManager *obj,
                    SaWManWindow                              *window,
                    SaWManWindow                              *relative,
                    SaWManWindowRelation                       relation );

DFBResult ISaWManManager_Real__RemoveWindow( SaWManManager *obj,
                    SaWManWindow                              *window );

DFBResult ISaWManManager_Real__SwitchFocus( SaWManManager *obj,
                    SaWManWindow                              *window );

DFBResult ISaWManManager_Real__SetScalingMode( SaWManManager *obj,
                    SaWManScalingMode                          mode );

DFBResult ISaWManManager_Real__SetWindowConfig( SaWManManager *obj,
                    SaWManWindow                              *window,
                    const SaWManWindowConfig                  *config,
                    SaWManWindowConfigFlags                    flags );

DFBResult ISaWManManager_Real__IsShowingWindow( SaWManManager *obj,
                    SaWManWindow                              *window,
                    DFBBoolean                                *ret_showing );

DFBResult ISaWManManager_Requestor__Activate( SaWManManager *obj
 );

DFBResult ISaWManManager_Requestor__QueueUpdate( SaWManManager *obj,
                    DFBWindowStackingClass                     stacking,
                    const DFBRegion                           *update );

DFBResult ISaWManManager_Requestor__ProcessUpdates( SaWManManager *obj,
                    DFBSurfaceFlipFlags                        flags );

DFBResult ISaWManManager_Requestor__CloseWindow( SaWManManager *obj,
                    SaWManWindow                              *window );

DFBResult ISaWManManager_Requestor__InsertWindow( SaWManManager *obj,
                    SaWManWindow                              *window,
                    SaWManWindow                              *relative,
                    SaWManWindowRelation                       relation );

DFBResult ISaWManManager_Requestor__RemoveWindow( SaWManManager *obj,
                    SaWManWindow                              *window );

DFBResult ISaWManManager_Requestor__SwitchFocus( SaWManManager *obj,
                    SaWManWindow                              *window );

DFBResult ISaWManManager_Requestor__SetScalingMode( SaWManManager *obj,
                    SaWManScalingMode                          mode );

DFBResult ISaWManManager_Requestor__SetWindowConfig( SaWManManager *obj,
                    SaWManWindow                              *window,
                    const SaWManWindowConfig                  *config,
                    SaWManWindowConfigFlags                    flags );

DFBResult ISaWManManager_Requestor__IsShowingWindow( SaWManManager *obj,
                    SaWManWindow                              *window,
                    DFBBoolean                                *ret_showing );


DFBResult SaWManManagerDispatch__Dispatch( SaWManManager *obj,
                    FusionID      caller,
                    int           method,
                    void         *ptr,
                    unsigned int  length,
                    void         *ret_ptr,
                    unsigned int  ret_size,
                    unsigned int *ret_length );


#endif
